package de.tudarmstadt.informatik.hostage.ui2.fragment.opengl;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

import android.opengl.GLES20;
import android.util.Log;

public class AnimatedMesh {
	private ByteBuffer data;
	
	private int vertexOffset;
	private int vertexSize;
	private int triangleOffset;
	private int triangleCount;
	
	private int vertexBuffer; // vbo
	private int indexBuffer;
	
	public AnimatedMesh(InputStream is) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			final int EOF = -1;
			int len;
			byte[] buffer = new byte[1 << 12];
			while (EOF != (len = is.read(buffer)))
				out.write(buffer, 0, len);
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}

		//data = ByteBuffer.wrap(out.toByteArray());
		data = ByteBuffer.allocateDirect(out.size());
		data.order(ByteOrder.nativeOrder());
		data.put(out.toByteArray());
		data.position(0);

		int magicNum = data.getInt();
		int version = data.getInt();
		
		vertexSize = 48;
		vertexOffset = data.getInt();
		int vertexCount = data.getInt();
		triangleOffset = data.getInt();
		triangleCount = data.getInt();
		int boneOffset = data.getInt();
		int boneCount = data.getInt();
		int actionOffset = data.getInt();
		int actionCount = data.getInt();
		
		IntBuffer buffers = IntBuffer.allocate(2);
		GLES20.glGenBuffers(2, buffers); // buffer names
		vertexBuffer = buffers.get();
		indexBuffer = buffers.get();
		
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, vertexBuffer);
		data.position(vertexOffset);
		GLES20.glBufferData(GLES20.GL_ARRAY_BUFFER, vertexSize * vertexCount, data.asFloatBuffer(), GLES20.GL_STATIC_DRAW);
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, 0);
		
		GLES20.glBindBuffer(GLES20.GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
		data.position(triangleOffset);
		GLES20.glBufferData(GLES20.GL_ELEMENT_ARRAY_BUFFER, 12 * triangleCount, data.asIntBuffer(), GLES20.GL_STATIC_DRAW);
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, 0);
	}
	
	public void draw(int program) {
		// TODO: cache attrib locations
		int positionIndex = GLES20.glGetAttribLocation(program, "position");
		int normalIndex = GLES20.glGetAttribLocation(program, "normal");
		
		GLES20.glEnableVertexAttribArray(positionIndex);
		GLES20.glEnableVertexAttribArray(normalIndex);
		
		//data.position(vertexOffset);
		//GLES20.glVertexAttribPointer(positionIndex, 3, GLES20.GL_FLOAT, false, vertexSize, data.asFloatBuffer());
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, vertexBuffer);
		GLES20.glVertexAttribPointer(positionIndex, 3, GLES20.GL_FLOAT, false, vertexSize, 0);
		GLES20.glVertexAttribPointer(normalIndex, 3, GLES20.GL_FLOAT, false, vertexSize, 12);
		//data.position(triangleOffset);
		//GLES20.glDrawElements(GLES20.GL_TRIANGLES, 3 * triangleCount, GLES20.GL_UNSIGNED_INT, data.asIntBuffer());
		GLES20.glBindBuffer(GLES20.GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
		GLES20.glDrawElements(GLES20.GL_TRIANGLES, 3 * triangleCount, GLES20.GL_UNSIGNED_INT, 0);
		GLES20.glBindBuffer(GLES20.GL_ELEMENT_ARRAY_BUFFER, 0);
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, 0);
		
		GLES20.glDisableVertexAttribArray(positionIndex);
		GLES20.glDisableVertexAttribArray(normalIndex);
	}
}
